/**
 * Available Bindings:
 *
 * sdc.state:
 *   A dict that is preserved between invocations
 *   of this script. Useful for caching bits of
 *   data e.g. counters.
 *
 * sdc.userParams:
 *   Key-value pairs specified in the ADVANCED tab
 *
 * sdc.log.<level>(msg, obj...):
 *   Use instead of println to send log messages to
 *   the log4j log instead of stdout. loglevel is
 *   any log4j level: e.g. info, warn, error, trace.
 *
 * sdc.isPreview():
 *   Determine if pipeline is in preview mode
 *
 * sdc.createMap(boolean listMap):
 *   Create a map for use as a field in a record.
 *   Pass true to this function to create a list map
 *   (ordered map)
 *
 * sdc.pipelineParameters():
 *   Map with pipeline runtime parameters.
 *
 * Available Constants:
 *  These are to assign a type to a field with a
 *  value null - sdc.NULL_BOOLEAN, sdc.NULL_CHAR, sdc.NULL_BYTE,
 *  sdc.NULL_SHORT, sdc.NULL_INTEGER, sdc.NULL_LONG,
 *  sdc.NULL_FLOATNULL_DOUBLE, sdc.NULL_DATE, sdc.NULL_DATETIME,
 *  sdc.NULL_TIME, sdc.NULL_DECIMAL, sdc.NULL_BYTE_ARRAY,
 *  sdc.NULL_STRING, sdc.NULL_LIST, sdc.NULL_MAP
 *
 * sdc.field:
 *   A Field object available to process. Contains
 *   value, fieldName, fieldPath, parentRecord,
 *   and attributes.
 *
 * sdc.field.parentRecord.attributes:
 *   a map of record header attributes.
 *
 * sdc.fieldOperator:
 *   An object that provides methods to handle
 *   replacing or removing the Field
 *
 *
 * Available Field Operator Methods:
 *
 * sdc.fieldOperator.updateField(Field):
 *   Replace the original field with the provided Field
 *
 * sdc.fieldOperator.dropField():
 *   Remove the original field from the parent record
 *
 * sdc.fieldOperator.updateRecordAttributes(Map<String, String>):
 *   Replace the contents of the record's attributes with
 *   the provided map
 *
 * sdc.fieldOperator.dropRecordAttributes():
 *   Remove the parent record's attributes
 *
 *  Please refrain from calling the fieldOperator
 *  methods more than once within a single execution
 *  of the protector script. Calling fieldOperator
 *  methods more than once results in undefined
 *  behavior. Calling none will result in the Field
 *  and parent Record remaining unchanged.
 **/

// Sample JavaScript code
try {
    // Change field value to a STRING value
    // sdc.field.value = 'Hello ' + i;

    // Change record root field value to a MAP
    // value and create an entry
    // sdc.field.value = { V : 'Hello' };

    // Access a MAP entry
    // sdc.field.value.X = field.value['V'] + ' World';

    // Modify a MAP entry
    // sdc.field.value.V = 5;

    // Create an ARRAY entry
    // sdc.field.value.A = ['Element 1', 'Element 2'];

    // Access a Array entry
    // sdc.field.value.B = sdc.field.value['A'][0];

    // Modify an existing ARRAY entry
    // sdc.field.value.A[0] = 100;

    // Assign a integer type to a field
    // and value null
    // sdc.field.value = sdc.NULL_INTEGER

    // Check if the field is NULL_INTEGER
    // If so, assign a value
    // if(sdc.field.value == sdc.NULL_INTEGER)
    //   sdc.field.value = 123

    // Modify a header attribute entry
    // sdc.field.attributes['name'] =
    //   sdc.field.attributes['first_name'] + ' ' +
    //   sdc.field.attributes['last_name']
    //

    // Get a record header with field names
    // ex. get sourceId and errorCode
    // var sourceId = sdc.field.parentRecord.sourceId
    // var errorCode = ''
    // if (sdc.field.parentRecord.errorCode) {
    //    errorCode = sdc.field.parentRecord.errorCode
    // }

    // Persist changes to the field
    sdc.fieldOperator.updateField(sdc.field)
    // Persist changes to the parent record's attributes
    sdc.fieldOperator.updateRecordAttributes(sdc.field.parentRecord.attributes)
} catch (e) {
    // Log errors
    sdc.log.error(String(e), e)
    throw e
}
